-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

USE Library
-- To create the Library Database unzip the backup and restore!
go
-------------------------------------------------------------------------------
-- A view which is a join
-------------------------------------------------------------------------------
CREATE VIEW dbo.BirthdayView
AS
SELECT m.member_no		AS [Member Number]
	, m.lastname			AS [Last Name] 
	, m.firstname			AS [First Name]
	, j.birth_date		AS [Birthday]
FROM dbo.member AS m 
	JOIN dbo.juvenile AS j ON m.member_no = j.member_no
go

-- SELECT the data
-- * is OK from a view as the view already limits the columns returned.
SELECT * FROM dbo.BirthdayView
go

-- Try an update... the error isn't great (because it's misleading) but
-- this won't work! Data Modifications through views can only impact
-- ONE of the base tables. If you change this to the transaction which 
-- follows it will work. However, you do have a more clever way to handle
-- this situation in SQL Server 2000 - INSTEAD OF triggers. See the 
-- example at the end of this script.
UPDATE BirthdayView
	SET [First Name] = 'Angie',
		Birthday = '7/24/1984'	
WHERE [Member Number] = 2
go

-- To correctly update this data you can only reference columns from ONE
-- of the base tables. This particular modification doesn't really need to 
-- be a transaction but many others might!
BEGIN TRAN
UPDATE BirthdayView
	SET [First Name] = 'Angie'
WHERE [Member Number] = 2
UPDATE BirthdayView
	SET Birthday = '7/24/1984'
WHERE [Member Number] = 2
COMMIT TRAN
go

-- Application developers/users will need to be aware of these restrictions/rules. 
-- To simplify the interface for users/applications consider using INSTEAD OF 
-- triggers! With an INSTEAD OF trigger you do NOT need to submit individual
-- statements - the trigger will direct the updates to the appropriate base tables.
go

-------------------------------------------------------------------------------
-- Views on Views
-------------------------------------------------------------------------------
SELECT bv.* 
FROM dbo.BirthdayView AS bv
WHERE bv.Birthday <= dateadd(yy, -16, getdate()+1)
go

CREATE VIEW dbo.SweetSixteen
AS
SELECT * 
FROM dbo.BirthdayView AS bv
WHERE bv.Birthday <= dateadd(yy, -16, getdate()+1)
go

SELECT * FROM dbo.SweetSixteen
go

sp_depends BirthdayView
exec sp_depends SweetSixteen
go
-- The following DROP will work even though there are dependent
-- Views. 
-- NOTE: You need to uncomment the command or highlight
-- and run.

-- DROP VIEW dbo.BirthdayView 

-- The base objects are not part of a "solid" dependency chain
-- unless the views are created with schemabinding. If schemabinding
-- is desired then ALL dependant objects in the chain must be dependant.

ALTER VIEW dbo.BirthdayView
	WITH SCHEMABINDING
AS
SELECT m.member_no	AS 'Member Number',
	m.lastname	AS 'Last Name', 
	m.firstname	AS 'First Name',
	j.birth_date	AS 'Birthday'
FROM dbo.member AS m 
	JOIN dbo.juvenile AS j ON m.member_no = j.member_no
go

ALTER VIEW dbo.SweetSixteen
	WITH SCHEMABINDING
AS
SELECT bv.* 
FROM dbo.BirthdayView AS bv
WHERE bv.Birthday <= dateadd(yy, -16, getdate()+1)
go

-------------------------------------------------------------------------------
-- INSTEAD OF Triggers allow you to direct the original modification
-- to multiple base tables...among other features!
-------------------------------------------------------------------------------

CREATE TRIGGER dbo.InsteadOfUpdateBirthdayView
ON dbo.BirthdayView
INSTEAD OF UPDATE
--with encryption
AS
DECLARE @RowCount	int
SELECT @RowCount = @@RowCount

IF @ROWCOUNT > 1
BEGIN
	RAISERROR('This trigger only handles SINGLE ROW modifications.', 16, 1)
	ROLLBACK TRAN
	RETURN
END
IF UPDATE([First Name]) 
BEGIN
	UPDATE M
		SET FirstName = i.[First Name]			
	FROM Inserted as I
		JOIN Member as M ON M.Member_no = i.[Member Number]
END
IF UPDATE([Last Name])
BEGIN
	UPDATE M
		SET LastName = i.[Last Name]
	FROM Inserted as I
		JOIN Member as M ON M.Member_no = i.[Member Number]
END
IF UPDATE([Birthday])
BEGIN
	UPDATE J
		SET J.birth_date = i.[Birthday]
	FROM Inserted as I
		JOIN Juvenile AS J ON J.Member_no = i.[Member Number]
END
go
